<?php

namespace June\Subscriber;

use June\Components\Api\Client\JuneApiClient;

use June\Extension\Checkout\Customer\CustomExtensionEntity;
use Shopware\Core\Checkout\Customer\CustomerEntity;
use Shopware\Core\Checkout\Order\Aggregate\OrderCustomer\OrderCustomerEntity;
use Shopware\Core\Checkout\Order\Event\OrderStateMachineStateChangeEvent;
use Shopware\Core\Content\Newsletter\Aggregate\NewsletterRecipient\NewsletterRecipientEntity;
use Shopware\Core\System\Country\CountryEntity;
use Shopware\Core\System\StateMachine\Event\StateMachineTransitionEvent;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;
use Shopware\Core\Checkout\Order\OrderEvents;
use Shopware\Core\Checkout\Cart\Event\CheckoutOrderPlacedEvent;
use Shopware\Core\Checkout\Cart\Event\CartChangedEvent;
use Shopware\Core\Checkout\Customer\Event\GuestCustomerRegisterEvent;
use Shopware\Core\Checkout\Customer\Event\CustomerRegisterEvent;

use Shopware\Core\Content\Newsletter\NewsletterEvents;
use Shopware\Core\Content\Newsletter\Event\NewsletterRegisterEvent;
use Shopware\Core\Content\Newsletter\Event\NewsletterConfirmEvent;
use Shopware\Core\Content\Newsletter\Event\NewsletterUnsubscribeEvent;
use Shopware\Core\Framework\DataAbstractionLayer\Event\EntityDeletedEvent;

use Shopware\Core\Checkout\Cart\LineItem\LineItem;
use Shopware\Core\Checkout\Order\OrderEntity;
use Shopware\Core\Checkout\Order\Aggregate\OrderLineItem\OrderLineItemEntity;

use Shopware\Core\Framework\DataAbstractionLayer\Event\EntityWrittenEvent;
use Shopware\Core\System\SalesChannel\Context\AbstractSalesChannelContextFactory;
use Shopware\Core\System\SalesChannel\Entity\SalesChannelRepository;
use Shopware\Core\System\SalesChannel\SalesChannelContext;
use Shopware\Core\System\SystemConfig\SystemConfigService;
use Shopware\Core\Framework\DataAbstractionLayer\EntityRepository;
use Shopware\Core\Framework\DataAbstractionLayer\Search\Filter\EqualsFilter;
use Shopware\Core\Framework\DataAbstractionLayer\Search\Criteria;
use Shopware\Core\Framework\Context;
use June\Components\Logger\Helper\LoggerHelper;
use function PHPUnit\Framework\isInstanceOf;

class JuneCommerceSubscriber implements EventSubscriberInterface
{
    const JUNE_COMMERCE_TOKEN = "June.config.commercetoken";
    const JUNE_COMMERCE_COLLECT_TOKEN = "June.config.commercecollecttoken";
    const JUNE_COMMERCE_COLLECT_AUTH = "June.config.commercecollectauth";
    const JUNE_TRANSFER_ORDER = "June.config.orderCommerceData";
    const JUNE_TRANSFER_FULFILLMENT = "June.config.fulfillCommerceData";
    const JUNE_TRANSFER_CART = "June.config.cartCommerceData";
    const JUNE_TRANSFER_GUESTDATA = "June.config.transferGuestData";
    const JUNE_TRANSFER_CUSTOMERS = "June.config.transferCustomers";
    const JUNE_TRANSFER_SUBSCRIBERS = "June.config.transferSubscribers";

    private EntityRepository $salesRepository;

    private EntityRepository $orderRepository;

    private EntityRepository $orderLineItemsRepository;

    private EntityRepository $languageRepository;

    private EntityRepository $localeRepository;

    private EntityRepository $countryRepository;

    private EntityRepository $customerEntity;

    private EntityRepository $newsletterRecipientRepository;

    private EntityRepository $customerTagsRepository;

    private SalesChannelRepository $categoryRepository;

    private EntityRepository $salutationRepository;

    private EntityRepository $juneCustomerRepository;

    private AbstractSalesChannelContextFactory $salesChannelContextFactory;

    private SystemConfigService $configService;

    private LoggerHelper $loggerHelper;

    private JuneApiClient $juneApiClient;

    private EntityRepository $stateMachineStateRepository;

    private EntityRepository $paymentMethodRepository;

    private array $loadedCustomerTags = [];

    public function __construct
    (
        EntityRepository $salesRepository,
        EntityRepository $orderRepository,
        EntityRepository $orderLineItemsRepository,
        EntityRepository $languageRepository,
        EntityRepository $localeRepository,
        EntityRepository $countryRepository,
        EntityRepository $customerEntity,
        EntityRepository $newsletterRecipientRepository,
        EntityRepository $customerTagsRepository,
        EntityRepository $salutationRepository,
        SalesChannelRepository $categoryRepository,
        EntityRepository $juneCustomerRepository,
        AbstractSalesChannelContextFactory $salesChannelContextFactory,
        SystemConfigService $configService,
        LoggerHelper $loggerHelper,
        JuneApiClient $juneApiClient,
        EntityRepository $stateMachineStateRepository,
        EntityRepository $paymentMethodRepository
    )
    {
        $this->salesRepository = $salesRepository;
        $this->orderRepository = $orderRepository;
        $this->orderLineItemsRepository = $orderLineItemsRepository;
        $this->languageRepository = $languageRepository;
        $this->localeRepository = $localeRepository;
        $this->countryRepository = $countryRepository;
        $this->customerEntity = $customerEntity;
        $this->newsletterRecipientRepository = $newsletterRecipientRepository;
        $this->customerTagsRepository = $customerTagsRepository;
        $this->salutationRepository = $salutationRepository;
        $this->categoryRepository = $categoryRepository;
        $this->juneCustomerRepository = $juneCustomerRepository;
        $this->salesChannelContextFactory = $salesChannelContextFactory;
        $this->configService = $configService;
        $this->loggerHelper = $loggerHelper;
        $this->juneApiClient = $juneApiClient;
        $this->stateMachineStateRepository = $stateMachineStateRepository;
        $this->paymentMethodRepository = $paymentMethodRepository;
    }

    public static function getSubscribedEvents(): array
    {
        return [
            CartChangedEvent::class => 'cartChanged',
            // CheckoutOrderPlacedEvent::EVENT_NAME => 'orderPlaced', // EVENT_NAME does not work? What is Shopware doing?
            CheckoutOrderPlacedEvent::class => 'orderPlaced',
            OrderEvents::ORDER_DELIVERY_WRITTEN_EVENT => 'orderFulfilled',
            GuestCustomerRegisterEvent::EVENT_NAME => 'guestRegister',
            CustomerRegisterEvent::EVENT_NAME => 'customerRegister',
            NewsletterEvents::NEWSLETTER_REGISTER_EVENT => 'newsletterRegister',
            NewsletterEvents::NEWSLETTER_CONFIRM_EVENT => 'newsletterSubscribe',
            NewsletterEvents::NEWSLETTER_UNSUBSCRIBE_EVENT => 'newsletterUnsubscribe',
            NewsletterEvents::NEWSLETTER_RECIPIENT_DELETED_EVENT => 'newsletterUnsubscribe',
            'state_enter.order_transaction.state.paid' => 'orderPaymentUpdated',

            # generic events for all entities
            //EntityWriteEvent::class => 'genericEntityWriteEvent',
        ];
    }

    /**
     * @param CartChangedEvent $event
     * @return void
     */
    public function cartChanged(CartChangedEvent $event) {
        $juneCommerceToken = $this->configService->get(self::JUNE_COMMERCE_TOKEN, $event->getContext()?->getSalesChannelId());
        if(!empty($juneCommerceToken)) {
            if($this->configService->get(self::JUNE_TRANSFER_CART, $event->getContext()?->getSalesChannelId())) {
                // make sure it's a registered customer, or we are supposed to transfer guests
                if(
                    $this->configService->get(self::JUNE_TRANSFER_GUESTDATA, $event->getContext()?->getSalesChannelId()) ||
                    !$event->getContext()?->getCustomer()?->getGuest()
                ) {
                    try {
                        $customer = $event->getContext()?->getCustomer();

                        // make sure we have enough customer information to use it
                        if ($customer?->getEmail()) {
                            $shopUrl = $this->getShopUrl($event->getContext()?->getSalesChannelId(), $event->getContext()?->getLanguageId());
                            $products = [];
                            $data = [];

                            if ($customer) {
                                $data['id'] = ($customer->getCustomerNumber() ?? $customer->getId());
                                $data['email'] = strtolower($customer->getEmail());
                                $data['customer'] = [
                                    'firstname' => $customer?->getFirstname(),
                                    'lastname' => $customer?->getLastname(),
                                    'store_id' => $customer?->getSalesChannelId(),
                                    '_locale' => $this->getLanguageCode($customer->getLanguageId()),
                                    'birthdate' => $customer?->getBirthday()->getTimestamp(),
                                    '_ba_title' => $customer->getTitle(),
                                    '_ba_first_name' => $customer?->getFirstname(),
                                    '_ba_last_name' => $customer?->getLastname(),
                                    '_ba_company_name' => $customer?->getCompany(),
                                    '_tags' => $customer?->getTags(),
                                    'guest' => $customer?->getGuest()
                                ];
                            } else {
                                $data['id'] = session_id();
                            }

                            foreach ($event->getCart()->getLineItems() as $item) {
                                if ($product = $this->transformLineItem($item, $shopUrl, $event->getContext()->getSalesChannelId(), $voucher)) {
                                    $products[] = $product;
                                }
                            }

                            $data['cart'] = [
                                'action' => 'update',
                                'products' => $products
                            ];

                            $this->juneApiClient->sendData($data, "https://engagement.juneapp.com/commerce/webhook/$juneCommerceToken", null);
                        }
                    } catch (\Exception $e) {
                        $this->loggerHelper->logError($e->getMessage(), "commerceCartChanged");
                    }
                }
            }
        } else {
            $this->loggerHelper->logError("The Commerce Token in the JUNE Plugin configuration is missing", "pluginConfig");
        }
    }

    /**
     * @param CheckoutOrderPlacedEvent $event
     * @return void
     */
    public function orderPlaced(CheckoutOrderPlacedEvent $event) {
        $order = $event->getOrder();

        $juneCommerceToken = $this->configService->get(self::JUNE_COMMERCE_TOKEN, $order->getSalesChannelId());
        if(!empty($juneCommerceToken)) {
            if($this->configService->get(self::JUNE_TRANSFER_ORDER, $order->getSalesChannelId())) {
                if(
                    $this->configService->get(self::JUNE_TRANSFER_GUESTDATA, $order->getSalesChannelId()) ||
                    !$order->getOrderCustomer()?->getCustomer()?->getGuest()
                ) {
                    $paymentCriteria = new Criteria($order->getTransactions()->getPaymentMethodIds());
                    $paymentMethod = $this->paymentMethodRepository->search($paymentCriteria, Context::createDefaultContext())->first();
                    if($paymentMethod->getTechnicalName() != "swag_paypal_paypal") {
                        try {
                            $voucher = 0;
                            $shopUrl = $this->getShopUrl($order->getSalesChannelId(), $order->getOrderCustomer()?->getCustomer()?->getLanguageId());

                            $recipient = array_keys($event->getMailStruct()->getRecipients());
                            $orderDate = $order->getOrderDateTime()->format('c');
                            $customer = $order->getOrderCustomer()->getCustomer() ?? $order->getOrderCustomer();
                            $products = [];

                            $address1 = "";
                            $address2 = "";
                            $city = "";
                            $zip_code = "";
                            $state = "";
                            $country_code = $this->getCountry($order->getSalesChannelId())->getIso();
                            $billingId = $order->getBillingAddressId();
                            foreach ($order->getAddresses() as $address) {
                                if($address->getId() == $billingId) {
                                    $address1 =$address?->getAdditionalAddressLine1();
                                    $address2 = $address?->getAdditionalAddressLine2();
                                    $zip_code = $address?->getZipcode();
                                    $city = $address?->getCity();
                                    $state = $address?->getCountryState()?->getName();
                                }
                            }

                            foreach ($order->getLineItems() as $item) {
                                // transform products and add voucher values to referenced variable
                                if ($product = $this->transformLineItem($item, $shopUrl, $order->getSalesChannelId(), $voucher)) {
                                    $products[] = $product;
                                }
                            }

                            $data = [
                                'action'    => 'order_placed',
                                'id'        => ($customer ? ($customer->getCustomerNumber() ?? $customer->getId()) : session_id()),
                                'email'     => $recipient[0],
                                'customer'  => [
                                    'firstname'     => $customer?->getFirstname(),
                                    'lastname'      => $customer?->getLastname(),
                                    'store_id'      => $customer?->getSalesChannelId(),
                                    '_locale'        => $this->getLanguageCode($customer->getLanguageId()),
                                    'birthdate'     => $customer?->getBirthday()->getTimestamp(),
                                    '_ba_title' => $customer->getTitle(),
                                    '_ba_first_name' => $customer?->getFirstname(),
                                    '_ba_last_name' => $customer?->getLastname(),
                                    '_ba_company_name' => $customer?->getCompany(),
                                    '_tags' => $customer?->getTags(),
                                    '_ba_address1' => $address1,
                                    '_ba_address2' => $address2,
                                    '_ba_zip_code' => $zip_code,
                                    '_ba_city' => $city,
                                    '_ba_state' => $state,
                                    '_ba_country_code' => $country_code,
                                    'guest'         => $customer?->getGuest()
                                ],
                                'order'     => [
                                    'id'            => $order->getOrderNumber(),
                                    'created_at'    => $orderDate,
                                    'fulfilled_at'  => null,
                                    'total'         => $order->getPrice()?->getTotalPrice(),
                                    'shipping'      => $order->getShippingCosts()?->getTotalPrice(),
                                    'voucher'       => $voucher,
                                    'products'      => $products,
                                ],
                            ];

                            // set new updatedAt date for customer so all properties are exported on the next schedule run
                            if ($this->configService->get(self::JUNE_TRANSFER_CUSTOMERS, $customer?->getSalesChannelId() ?? $order->getSalesChannelId())) {
                                $this->createJuneCustomer($customer, $customer?->getSalesChannelId() ?? $order->getSalesChannelId(), $this->getLanguageCode($customer->getLanguageId()));

                                if ($customer instanceof CustomerEntity) {
                                    $this->customerEntity->update([
                                        [
                                            'id' => $customer->getId()
                                        ]
                                    ], $event->getContext());
                                }
                            }

                            // send order information
                            $this->juneApiClient->sendData($data,"https://engagement.juneapp.com/commerce/webhook/$juneCommerceToken", null);
                        } catch(\Exception $e) {
                            $this->loggerHelper->logError($e->getMessage()." - ".$e->getTraceAsString(), "commerceOrderPlaced");
                        }
                    }
                }
            }
        } else {
            $this->loggerHelper->logError("The Commerce Token in the JUNE Plugin configuration is missing", "pluginConfig");
        }
    }

    /**
     * @param EntityWrittenEvent $event
     * @return void
     */
    public function orderFulfilled(EntityWrittenEvent $event): void {

        try {
            $event_data = $event->getPayloads();
            if (isset($event_data[0]["orderId"])) {
                $orderId = $event_data[0]["orderId"];
                $order = $this->getOrder($orderId, Context::createDefaultContext());
                if ($order) {
                    $juneCommerceToken = $this->configService->get(self::JUNE_COMMERCE_TOKEN, $order->getSalesChannelId());
                    if(!empty($juneCommerceToken)) {
                        if ($this->configService->get(self::JUNE_TRANSFER_FULFILLMENT, $order->getSalesChannelId())) {

                            $customer = $order->getOrderCustomer()?->getCustomer() ?? $order->getOrderCustomer();
                            $products = [];

                            $itemCriteria = new Criteria();
                            $itemCriteria->addFilter(new EqualsFilter("orderId", $order->getId()));
                            $lineItems = $this->orderLineItemsRepository->search($itemCriteria, Context::createDefaultContext())->getElements();

                            $orderStateCriteria = new Criteria([$order->getStateId()]);
                            $orderState = $this->stateMachineStateRepository->search($orderStateCriteria, Context::createDefaultContext())->get($order->getStateId());

                            if ($orderState->getTechnicalName() === "completed") {
                                $voucher = 0;
                                $shopUrl = $this->getShopUrl($order->getSalesChannelId(), $order->getLanguageId());
                                $orderDate = $order->getOrderDateTime()->format('c');

                                $address1 = "";
                                $address2 = "";
                                $city = "";
                                $zip_code = "";
                                $state = "";
                                $country_code = $this->getCountry($order->getSalesChannelId())->getIso();
                                $billingId = $order->getBillingAddressId();
                                foreach ($order->getAddresses() as $address) {
                                    if($address->getId() == $billingId) {
                                        $address1 =$address?->getAdditionalAddressLine1();
                                        $address2 = $address?->getAdditionalAddressLine2();
                                        $zip_code = $address?->getZipcode();
                                        $city = $address?->getCity();
                                        $state = $address?->getCountryState()?->getName();
                                    }
                                }

                                foreach ($lineItems as $item) {
                                    if ($product = $this->transformLineItem($item, $shopUrl, $order->getSalesChannelId(), $voucher)) {
                                        $products[] = $product;
                                    }
                                }

                                $data = [
                                    'action'    => 'order_fulfilled',
                                    'id'        => ($customer ? ($customer->getCustomerNumber() ?? $customer->getId()) : session_id()),
                                    'email'     => strtolower($customer?->getEmail()),
                                    'customer'  => [
                                        'firstname'     => $customer?->getFirstname(),
                                        'lastname'      => $customer?->getLastname(),
                                        'store_id'      => $customer?->getSalesChannelId(),
                                        '_locale'        => $this->getLanguageCode($customer->getLanguageId()),
                                        'birthdate'     => $customer?->getBirthday()->getTimestamp(),
                                        '_ba_title' => $customer->getTitle(),
                                        '_ba_first_name' => $customer?->getFirstname(),
                                        '_ba_last_name' => $customer?->getLastname(),
                                        '_ba_company_name' => $customer?->getCompany(),
                                        '_tags' => $customer?->getTags(),
                                        '_ba_address1' => $address1,
                                        '_ba_address2' => $address2,
                                        '_ba_zip_code' => $zip_code,
                                        '_ba_city' => $city,
                                        '_ba_state' => $state,
                                        '_ba_country_code' => $country_code,
                                        'guest'         => $customer?->getGuest()
                                    ],
                                    'fulfillment' => [
                                        'id'            => $order->getOrderNumber(),
                                        'created_at'    => $orderDate,
                                        'fulfilled_at'  => null,
                                        'total'         => $order->getPrice()?->getTotalPrice(),
                                        'shipping'      => $order->getShippingCosts()?->getTotalPrice(),
                                        'voucher'       => $voucher,
                                        'products'      => $products,
                                    ],
                                ];

                                if ($order->getDeliveries()) {
                                    foreach ($order->getDeliveries() AS $delivery) {
                                        $deliveryStateCriteria = new Criteria([$delivery->getStateId()]);
                                        $deliveryState = $this->stateMachineStateRepository->search($deliveryStateCriteria, Context::createDefaultContext())->get($delivery->getStateId());

                                        if ($deliveryState->getTechnicalName() === 'shipped') {
                                            $data['fulfillment']['fulfilled_at'] = $delivery->getUpdatedAt()->format('c');
                                        }
                                    }
                                }

                                // set new updatedAt date for customer so all properties are exported on the next schedule run
                                if ($this->configService->get(self::JUNE_TRANSFER_CUSTOMERS, $order->getSalesChannelId())) {
                                    $this->createJuneCustomer($customer, $order->getSalesChannelId(), $this->getLanguageCode($order->getLanguageId()));

                                    // trigger update to make sure customer is exported in schedule
                                    if ($customer instanceof CustomerEntity) {
                                        $this->customerEntity->update([
                                            [
                                                'id' => $customer->getId()
                                            ]
                                        ], $event->getContext());
                                    }
                                }

                                $this->juneApiClient->sendData($data, "https://engagement.juneapp.com/commerce/webhook/$juneCommerceToken", null);
                            }
                        }
                    } else {
                        $this->loggerHelper->logError("The Commerce Token in the JUNE Plugin configuration is missing", "pluginConfig");
                    }
                }
            }
        } catch (\Exception $e) {
            $this->loggerHelper->logError($e->getMessage(), "commerceOrderFulfilled");
        }
    }

    /**
     * @param EntityWrittenEvent $event
     * @return void
     */
    public function orderPaymentUpdated(OrderStateMachineStateChangeEvent $event): void {
        try {
            $order = $event->getOrder();
            if ($order) {
                if($order->getTransactions()->first()->getPaymentMethod()->getTechnicalName() == "swag_paypal_paypal") {
                    $orderStateCriteria = new Criteria([$order->getTransactions()?->first()->getStateId()]);
                    $orderState = $this->stateMachineStateRepository->search($orderStateCriteria, Context::createDefaultContext())->first();
                    $juneCommerceToken = $this->configService->get(self::JUNE_COMMERCE_TOKEN, $order->getSalesChannelId());
                    if (!empty($juneCommerceToken)) {
                        if ($this->configService->get(self::JUNE_TRANSFER_ORDER, $order->getSalesChannelId())) {
                            $voucher = 0;
                            $shopUrl = $this->getShopUrl($order->getSalesChannelId(), $order->getOrderCustomer()?->getCustomer()?->getLanguageId());

                            $recipient = array_keys($event->getMailStruct()->getRecipients());
                            $orderDate = $order->getOrderDateTime()->format('c');
                            $customer = $order->getOrderCustomer()->getCustomer() ?? $order->getOrderCustomer();
                            $products = [];

                            $address1 = "";
                            $address2 = "";
                            $city = "";
                            $zip_code = "";
                            $state = "";
                            $country_code = $this->getCountry($order->getSalesChannelId())->getIso();
                            $billingId = $order->getBillingAddressId();
                            foreach ($order->getAddresses() as $address) {
                                if($address->getId() == $billingId) {
                                    $address1 =$address?->getAdditionalAddressLine1();
                                    $address2 = $address?->getAdditionalAddressLine2();
                                    $zip_code = $address?->getZipcode();
                                    $city = $address?->getCity();
                                    $state = $address?->getCountryState()?->getName();
                                }
                            }

                            foreach ($order->getLineItems() as $item) {
                                // transform products and add voucher values to referenced variable
                                if ($product = $this->transformLineItem($item, $shopUrl, $order->getSalesChannelId(), $voucher)) {
                                    $products[] = $product;
                                }
                            }

                            $data = [
                                'action'    => 'order_placed',
                                'id'        => ($customer ? ($customer->getCustomerNumber() ?? $customer->getId()) : session_id()),
                                'email'     => $recipient[0],
                                'customer'  => [
                                    'firstname'     => $customer?->getFirstname(),
                                    'lastname'      => $customer?->getLastname(),
                                    'store_id'      => $customer?->getSalesChannelId(),
                                    '_locale'        => $this->getLanguageCode($customer->getLanguageId()),
                                    'birthdate'     => $customer?->getBirthday()->getTimestamp(),
                                    '_ba_title' => $customer->getTitle(),
                                    '_ba_first_name' => $customer?->getFirstname(),
                                    '_ba_last_name' => $customer?->getLastname(),
                                    '_ba_company_name' => $customer?->getCompany(),
                                    '_tags' => $customer?->getTags(),
                                    '_ba_address1' => $address1,
                                    '_ba_address2' => $address2,
                                    '_ba_zip_code' => $zip_code,
                                    '_ba_city' => $city,
                                    '_ba_state' => $state,
                                    '_ba_country_code' => $country_code,
                                    'guest'         => $customer?->getGuest()
                                ],
                                'order'     => [
                                    'id'            => $order->getOrderNumber(),
                                    'created_at'    => $orderDate,
                                    'fulfilled_at'  => null,
                                    'total'         => $order->getPrice()?->getTotalPrice(),
                                    'shipping'      => $order->getShippingCosts()?->getTotalPrice(),
                                    'voucher'       => $voucher,
                                    'products'      => $products,
                                ],
                            ];

                            // set new updatedAt date for customer so all properties are exported on the next schedule run
                            if ($this->configService->get(self::JUNE_TRANSFER_CUSTOMERS, $customer?->getSalesChannelId() ?? $order->getSalesChannelId())) {
                                $this->createJuneCustomer($customer, $customer?->getSalesChannelId() ?? $order->getSalesChannelId(), $this->getLanguageCode($customer->getLanguageId()));

                                if ($customer instanceof CustomerEntity) {
                                    $this->customerEntity->update([
                                        [
                                            'id' => $customer->getId()
                                        ]
                                    ], $event->getContext());
                                }
                            }

                            // send order information
                            $this->juneApiClient->sendData($data,"https://engagement.juneapp.com/commerce/webhook/$juneCommerceToken", null);
                        }
                    }
                }
            }
        } catch (\Exception $e) {
            $this->loggerHelper->logError($e->getMessage(), "commerceOrderPaymentUpdated");
        }
    }

    /**
     * @param OrderLineItemEntity|LineItem $item
     * @param $shopUrl
     * @param $salesChannelId
     * @param $voucher
     * @return array|void
     */
    private function transformLineItem(OrderLineItemEntity|LineItem $item, $shopUrl, $salesChannelId, &$voucher) {
        try {
            if ($item->getType() === "promotion") {
                $voucher = $voucher + round($item instanceof LineItem ? $item?->getPrice()?->getTotalPrice() : $item->getTotalPrice(), 2);
            }

            $productData = $item->getPayload();

            $categories = [];
            if (!empty($productData['categoryIds'])) {
                foreach ($productData['categoryIds'] as $categoryId) {
                    if ($categoryId) {
                        $categoryCriteria = new Criteria();
                        $categoryCriteria = $categoryCriteria->addFilter(new EqualsFilter('id', $categoryId));
                        $category = $this->categoryRepository->search($categoryCriteria, $this->createSalesChannelContext($salesChannelId))->first();
                        if ($category?->getName()) {
                            $categories[] = $category->getName();
                        }
                    }
                }
            }

            $productId = ($item instanceof LineItem ? $item->getId() : $item->getProductId());
            $productExternalIdentifier = !empty($productData['productNumber']) ? $productData['productNumber'] : $item->getId();
            $productPrice = ($item instanceof LineItem ? $item?->getPrice()?->getUnitPrice() : $item->getUnitPrice());

            // load variant definitions for cart items
            $productName = $item->getLabel();
            if (!empty($productData['options'])) {
                foreach ($productData['options'] as $optionItem) {
                    if (!empty($optionItem['option'])) {
                        $productName .= " | " . $optionItem['option'];
                    }
                }
            }

            $product = [
                'id' => $productExternalIdentifier,
                'name' => $productName,
                'qty' => $item->getQuantity(),
                'price' => $productPrice,
                'category' => $categories
            ];

            if ($item->getCover() != NULL) {
                $product['image'] = $item?->getCover()?->getUrl();
            }

            if ($shopUrl && $productId) {
                // $item->getProduct()->getSeoUrls();
                $product['link'] = $shopUrl . "/detail/" . $productId;
            }

            return $product;
        } catch (\Exception $e) {
            $this->loggerHelper->logError($e->getMessage(), "commerceTransformLineItems");
        }
    }

    /**
     * @param GuestCustomerRegisterEvent $event
     * @return void
     */
    public function guestRegister(GuestCustomerRegisterEvent $event) {
        if($this->configService->get(self::JUNE_TRANSFER_GUESTDATA, $event->getSalesChannelId())) {
            $this->createJuneCustomer($event->getCustomer(), $event->getSalesChannelId(), $this->getLanguageCode($event->getSalesChannelContext()->getLanguageId()));
        }
    }

    /**
     * @param CustomerRegisterEvent $event
     * @return void
     */
    public function customerRegister(CustomerRegisterEvent $event) {
        $this->createJuneCustomer($event->getCustomer(), $event->getSalesChannelId(), $this->getLanguageCode($event->getSalesChannelContext()->getLanguageId()));
    }

    /**
     * @param CustomerEntity|OrderCustomerEntity $customer
     * @param $salesChannelId
     * @param $locale
     * @return void
     */
    public function createJuneCustomer(CustomerEntity|OrderCustomerEntity $customer, $salesChannelId, $locale):void {
        if ($this->configService->get(self::JUNE_TRANSFER_CUSTOMERS, $salesChannelId)) {
            try {
                $commerceCollectToken = $this->configService->get(JuneCommerceSubscriber::JUNE_COMMERCE_COLLECT_TOKEN, $salesChannelId);
                $commerceCollectAuth = $this->configService->get(JuneCommerceSubscriber::JUNE_COMMERCE_COLLECT_AUTH, $salesChannelId);

                if ($customer->getEmail() && $commerceCollectToken && $commerceCollectAuth) {
                    $customerUpdateFields = [
                        "external_id",
                        "firstname",
                        "lastname",
                        "salutation",
                        "salutation_readable",
                        "locale",
                        "updated_at",
                        "store_id",
                        "guest"
                    ];

                    $subscriber = $this->getNewsletterRecipientByEmail($customer->getEmail());

                    $salutation = $customer->getSalutation();
                    if (!$salutation && $customer->getSalutationId()) {
                        $salutation = $this->getSalutation($customer->getSalutationId());
                    }

                    $data = [
                        "customer"  => [
                            "external_id"           => $customer->getCustomerNumber() ?? $customer->getId(),
                            "email"                 => strtolower($customer->getEmail()),
                            "firstname"             => $customer->getFirstName(),
                            "lastname"              => $customer->getLastName(),
                            "salutation"            => $salutation?->getSalutationKey(),
                            "salutation_readable"   => $salutation?->getLetterName(),
                            "store_id"              => $salesChannelId,
                            "locale"                => $locale,
                            "created_at"            => $customer->getCreatedAt()?->format('c') ?? $subscriber?->getCreatedAt()?->format('c'),
                            "updated_at"            => $customer->getUpdatedAt()?->format('c') ?? $subscriber?->getUpdatedAt()?->format('c'),
                            "guest"                 => true
                        ]
                    ];

                    if ($customer instanceof CustomerEntity) {
                        if ($customer?->getId()) {
                            $data["customer"]['customer_id'] = $customer->getId();
                            $customerUpdateFields[] = 'customer_id';
                        }

                        $data["customer"]["geo_location"] = [
                            "city"                  => $customer->getDefaultBillingAddress()?->getCity(),
                            "continent"             => "",
                            "country"               => $customer->getDefaultBillingAddress()?->getCountry()?->getName()
                        ];

                        $data["customer"]["guest"] = $customer->getGuest();

                        if ($subscriber?->getConfirmedAt()) {
                            $data['customer']['subscription_confirmed_at'] = $subscriber->getConfirmedAt()->format('c');
                            $customerUpdateFields[] = 'subscription_confirmed_at';
                        }
                        if ($customer->getBirthday()) {
                            $data['customer']['birthday'] = $customer->getBirthday()->format('c');
                            if (!in_array('birthday', $customerUpdateFields)) {
                                $customerUpdateFields[] = 'birthday';
                            }
                        }
                        if ($customer->getLastLogin()) {
                            $data['customer']['last_login'] = $customer->getLastLogin()->format('c');
                            if (!in_array('last_login', $customerUpdateFields)) {
                                $customerUpdateFields[] = 'last_login';
                            }
                        }

                        if ($customer->getLastOrderDate()) {
                            $data['customer']['last_order_date'] = $customer->getLastOrderDate()->format('c');
                            if (!in_array('last_order_date', $customerUpdateFields)) {
                                $customerUpdateFields[] = 'last_order_date';
                            }
                        }
                        if ($customer->getOrderCount()) {
                            $data['customer']['order_count'] = $customer->getOrderCount();
                            if (!in_array('order_count', $customerUpdateFields)) {
                                $customerUpdateFields[] = 'order_count';
                            }
                        }
                        if ($customer->getOrderTotalAmount()) {
                            $data['customer']['order_total_amount'] = $customer->getOrderTotalAmount();
                            if (!in_array('order_total_amount', $customerUpdateFields)) {
                                $customerUpdateFields[] = 'order_total_amount';
                            }
                        }

                        if ($customer->getCampaignCode()) {
                            $data['customer']['campaign_code'] = $customer->getCampaignCode();
                            if (!in_array('campaign_code', $customerUpdateFields)) {
                                $customerUpdateFields[] = 'campaign_code';
                            }
                        }
                        if ($customer->getAffiliateCode()) {
                            $data['customer']['affiliate_code'] = $customer->getAffiliateCode();
                            if (!in_array('affiliate_code', $customerUpdateFields)) {
                                $customerUpdateFields[] = 'affiliate_code';
                            }
                        }

                        if ($customer->getTagIds()) {
                            $data['customer']['_tags'] = [];
                            foreach ($customer->getTagIds() as $tagId) {
                                if ($tag = $this->getTag($tagId)) {
                                    $data['customer']['_tags'][] = $tag;
                                }
                            }
                        }
                    }

                    // add more information to customer if available
                    if ($customer->getCustomFields()) {
                        foreach ($customer->getCustomFields() as $fieldName => $fieldValue) {
                            if (!isset($data['customer'][$fieldName])) {
                                $transformedFieldName = ($data['customer']['guest'] ? 'guest_' : '') . $fieldName;
                                $data['customer'][$transformedFieldName] = $fieldValue;

                                if (!in_array($transformedFieldName, $customerUpdateFields)) {
                                    $customerUpdateFields[] = $transformedFieldName;
                                }
                            }
                        }
                    }

                    $data['customer']['_unsubscribed'] = !($subscriber?->getStatus() === "direct" || $subscriber?->getStatus() === "optIn");
                    if (!in_array('_unsubscribed', $customerUpdateFields)) {
                        $customerUpdateFields[] = '_unsubscribed';
                    }

                    $data['customer']['intent'] = $subscriber?->getStatus();
                    if (!in_array('intent', $customerUpdateFields)) {
                        $customerUpdateFields[] = 'intent';
                    }

                    if ($subscriber?->getCity()) {
                        $data['customer']['city'] = $subscriber->getCity();
                        $data['customer']['geo_location']['city'] = $subscriber->getCity();
                        if (!in_array('city', $customerUpdateFields)) {
                            $customerUpdateFields[] = 'city';
                        }
                    }
                    if ($subscriber?->getZipCode()) {
                        $data['customer']['zip_code'] = $subscriber->getZipCode();
                        if (!in_array('zip_code', $customerUpdateFields)) {
                            $customerUpdateFields[] = 'zip_code';
                        }
                    }

                    if ($subscriber?->getTags()) {
                        if (!isset($data['customer']['_tags'])) {
                            $data['customer']['_tags'] = [];
                        }
                        foreach ($subscriber->getTags() as $tag) {
                            if ($tag?->getName() && !in_array($tag->getName(), $data['customer']['_tags'])) {
                                $data['customer']['_tags'][] = $tag->getName();
                            }
                        }
                    }

                    /* dont use default import because it relies on queue times
                    $requestData = [
                        "action" => "customer_create",
                        "json" => [
                            $data['customer']
                        ],
                        "activate_trigger" => true,
                        "default_subscribe" => false,
                        "force_subscribe" => false,
                        "update_if_exists" => true,
                        "overwrite_tags" => true,
                        "update_fields" => $customerUpdateFields
                    ];
                    $this->juneApiClient->sendData($requestData, "https://engagement.juneapp.com/collect/$commerceCollectToken/import/json", $commerceCollectAuth);
                    */

                    $this->upsertCustomer($customer, $data['customer'], $commerceCollectToken, $commerceCollectAuth, $salesChannelId);
                }
            } catch (\Exception $e) {
                $this->loggerHelper->logError($e->getMessage(), "commerceCustomerCreate");
            }
        }
    }

    /**
     * @param NewsletterRegisterEvent $event
     * @return void
     */
    public function newsletterRegister(NewsletterRegisterEvent $event) {
        $salesChannelId = $event->getSalesChannelId();
        if ($this->configService->get(self::JUNE_TRANSFER_SUBSCRIBERS, $salesChannelId)) {
            try {
                $commerceCollectToken = $this->configService->get(self::JUNE_COMMERCE_COLLECT_TOKEN, $salesChannelId);
                $commerceCollectAuth = $this->configService->get(self::JUNE_COMMERCE_COLLECT_AUTH, $salesChannelId);

                // $newsletterRecipient = $event->getNewsletterRecipient(); // The event newsletterRecipient does not contain any tags or tagIds...
                $subscriber = $this->getNewsletterRecipient($event->getNewsletterRecipientId());

                if ($subscriber && $subscriber->getEmail() && !empty($commerceCollectToken) && !empty($commerceCollectAuth)) {
                    $customerUpdateFields = [
                        "firstname",
                        "lastname",
                        "locale",
                        "_unsubscribed"
                    ];

                    $customer = $this->getCustomerByEmail($subscriber->getEmail());

                    $enrichedCustomer = [
                        'external_id'   => $customer?->getCustomerNumber() ?? $subscriber?->getId(),
                        'email'         => strtolower($subscriber->getEmail()),
                        'firstname'     => $subscriber->getFirstname(),
                        'lastname'      => $subscriber->getLastname(),
                        'store_id'      => $salesChannelId,
                        'locale'        => $this->getLanguageCode($event->getContext()->getLanguageId()),
                        'guest'         => $customer?->getGuest() ?? false,
                        '_unsubscribed' => true,
                        'geo_location'  => []
                    ];

                    // add more information to customer if available
                    if ($subscriber?->getConfirmedAt()) {
                        $enrichedCustomer['subscription_confirmed_at'] = $subscriber->getConfirmedAt()->format('c');
                        $customerUpdateFields[] = 'subscription_confirmed_at';
                    }

                    if ($customer?->getId()) {
                        $enrichedCustomer['customer_id'] = $customer->getId();
                        $customerUpdateFields[] = 'customer_id';
                    }

                    if ($customer?->getCustomFields()) {
                        foreach ($customer->getCustomFields() as $fieldName => $fieldValue) {
                            if (!isset($enrichedCustomer[$fieldName])) {
                                $transformedFieldName = ($enrichedCustomer['guest'] ? 'guest_' : '') . $fieldName;
                                $enrichedCustomer[$transformedFieldName] = $fieldValue;

                                if (!in_array($transformedFieldName, $customerUpdateFields)) {
                                    $customerUpdateFields[] = $transformedFieldName;
                                }
                            }
                        }
                    }
                    if ($customer?->getBirthday()) {
                        $enrichedCustomer['birthday'] = $customer->getBirthday()->format('c');
                        if (!in_array('birthday', $customerUpdateFields)) {
                            $customerUpdateFields[] = 'birthday';
                        }
                    }
                    if ($customer?->getLastLogin()) {
                        $enrichedCustomer['last_login'] = $customer->getLastLogin()->format('c');
                        if (!in_array('last_login', $customerUpdateFields)) {
                            $customerUpdateFields[] = 'last_login';
                        }
                    }
                    if ($customer?->getLastOrderDate()) {
                        $enrichedCustomer['last_order_date'] = $customer->getLastOrderDate()->format('c');
                        if (!in_array('last_order_date', $customerUpdateFields)) {
                            $customerUpdateFields[] = 'last_order_date';
                        }
                    }
                    if ($customer?->getOrderCount()) {
                        $enrichedCustomer['order_count'] = $customer->getOrderCount();
                        if (!in_array('order_count', $customerUpdateFields)) {
                            $customerUpdateFields[] = 'order_count';
                        }
                    }
                    if ($customer?->getOrderTotalAmount()) {
                        $enrichedCustomer['order_total_amount'] = $customer->getOrderTotalAmount();
                        if (!in_array('order_total_amount', $customerUpdateFields)) {
                            $customerUpdateFields[] = 'order_total_amount';
                        }
                    }

                    if ($customer?->getCampaignCode()) {
                        $enrichedCustomer['campaign_code'] = $customer->getCampaignCode();
                        if (!in_array('campaign_code', $customerUpdateFields)) {
                            $customerUpdateFields[] = 'campaign_code';
                        }
                    }

                    if ($customer?->getAffiliateCode()) {
                        $enrichedCustomer['affiliate_code'] = $customer->getAffiliateCode();
                        if (!in_array('affiliate_code', $customerUpdateFields)) {
                            $customerUpdateFields[] = 'affiliate_code';
                        }
                    }

                    if ($customer?->getGroup()?->getName()) {
                        $enrichedCustomer['customer_group'] = $customer->getGroup()->getName();
                        if (!in_array('customer_group', $customerUpdateFields)) {
                            $customerUpdateFields[] = 'customer_group';
                        }
                    }

                    $enrichedCustomer['_unsubscribed'] = !($subscriber?->getStatus() === "direct" || $subscriber?->getStatus() === "optIn");
                    if (!in_array('_unsubscribed', $customerUpdateFields)) {
                        $customerUpdateFields[] = '_unsubscribed';
                    }

                    $enrichedCustomer['intent'] = $subscriber?->getStatus();
                    if (!in_array('intent', $customerUpdateFields)) {
                        $customerUpdateFields[] = 'intent';
                    }

                    if ($subscriber?->getCity()) {
                        $enrichedCustomer['city'] = $subscriber->getCity();
                        $enrichedCustomer['geo_location']['city'] = $subscriber->getCity();
                        if (!in_array('city', $customerUpdateFields)) {
                            $customerUpdateFields[] = 'city';
                        }
                    }
                    if ($subscriber?->getZipCode()) {
                        $enrichedCustomer['zip_code'] = $subscriber->getZipCode();
                        if (!in_array('zip_code', $customerUpdateFields)) {
                            $customerUpdateFields[] = 'zip_code';
                        }
                    }

                    if ($customer?->getTagIds()) {
                        $enrichedCustomer['_tags'] = [];
                        foreach ($customer->getTagIds() as $tagId) {
                            if ($tag = $this->getTag($tagId)) {
                                $enrichedCustomer['_tags'][] = $tag;
                            }
                        }
                    }
                    if ($subscriber?->getTags()) {
                        if (!isset($enrichedCustomer['_tags'])) {
                            $enrichedCustomer['_tags'] = [];
                        }
                        foreach ($subscriber->getTags() as $tag) {
                            if ($tag?->getName() && !in_array($tag->getName(), $enrichedCustomer['_tags'])) {
                                $enrichedCustomer['_tags'][] = $tag->getName();
                            }
                        }
                    }

                    /* dont use default import because it relies on queue times
                    $requestData = [
                        "action" => "subscription_create",
                        "json" => [$enrichedCustomer],
                        "activate_trigger" => true,
                        "default_subscribe" => false,
                        "force_subscribe" => false,
                        "update_if_exists" => true,
                        "update_fields" => $customerUpdateFields
                    ];
                    $this->juneApiClient->sendData($requestData, "https://engagement.juneapp.com/collect/$commerceCollectToken/import/json", $commerceCollectAuth);
                    */
                    $this->upsertCustomer($customer, $enrichedCustomer, $commerceCollectToken, $commerceCollectAuth, $salesChannelId);
                }
            } catch (\Exception $e) {
                $this->loggerHelper->logError($e->getMessage(), "commerceNewsletterRegister");
            }
        }
    }

    /**
     * @param NewsletterConfirmEvent $event
     * @return void
     */
    public function newsletterSubscribe(NewsletterConfirmEvent $event) {
        $salesChannelId = $event->getSalesChannelId();
        if ($this->configService->get(self::JUNE_TRANSFER_SUBSCRIBERS, $salesChannelId)) {
            try {
                $commerceCollectToken = $this->configService->get(self::JUNE_COMMERCE_COLLECT_TOKEN, $salesChannelId);
                $commerceCollectAuth = $this->configService->get(self::JUNE_COMMERCE_COLLECT_AUTH, $salesChannelId);

                // $subscriber = $event->getNewsletterRecipient(); // The event newsletterRecipient does not contain any tags or tagIds...
                $subscriber = $this->getNewsletterRecipient($event->getNewsletterRecipientId());

                if ($subscriber && $subscriber->getEmail() && !empty($commerceCollectToken) && !empty($commerceCollectAuth)) {
                    $customerUpdateFields = [
                        "firstname",
                        "lastname",
                        "locale",
                        "_unsubscribed"
                    ];

                    $customer = $this->getCustomerByEmail($subscriber->getEmail());

                    $enrichedCustomer = [
                        'external_id'   => $customer?->getCustomerNumber() ?? $subscriber?->getId(),
                        'email'         => strtolower($subscriber->getEmail()),
                        'firstname'     => $subscriber->getFirstname(),
                        'lastname'      => $subscriber->getLastname(),
                        'store_id'      => $salesChannelId,
                        'locale'        => $this->getLanguageCode($event->getContext()->getLanguageId()),
                        'guest'         => $customer?->getGuest() ?? false,
                        '_unsubscribed' => false,
                        'geo_location'  => []
                    ];

                    // add more information to customer if available
                    if ($subscriber?->getConfirmedAt()) {
                        $enrichedCustomer['subscription_confirmed_at'] = $subscriber->getConfirmedAt()->format('c');
                        $customerUpdateFields[] = 'subscription_confirmed_at';
                    }

                    if ($customer?->getId()) {
                        $enrichedCustomer['customer_id'] = $customer->getId();
                        $customerUpdateFields[] = 'customer_id';
                    }

                    if ($customer?->getCustomFields()) {
                        foreach ($customer->getCustomFields() as $fieldName => $fieldValue) {
                            if (!isset($enrichedCustomer[$fieldName])) {
                                $transformedFieldName = ($enrichedCustomer['guest'] ? 'guest_' : '') . $fieldName;
                                $enrichedCustomer[$transformedFieldName] = $fieldValue;

                                if (!in_array($transformedFieldName, $customerUpdateFields)) {
                                    $customerUpdateFields[] = $transformedFieldName;
                                }
                            }
                        }
                    }
                    if ($customer?->getBirthday()) {
                        $enrichedCustomer['birthday'] = $customer->getBirthday()->format('c');
                        if (!in_array('birthday', $customerUpdateFields)) {
                            $customerUpdateFields[] = 'birthday';
                        }
                    }
                    if ($customer?->getLastLogin()) {
                        $enrichedCustomer['last_login'] = $customer->getLastLogin()->format('c');
                        if (!in_array('last_login', $customerUpdateFields)) {
                            $customerUpdateFields[] = 'last_login';
                        }
                    }
                    if ($customer?->getLastOrderDate()) {
                        $enrichedCustomer['last_order_date'] = $customer->getLastOrderDate()->format('c');
                        if (!in_array('last_order_date', $customerUpdateFields)) {
                            $customerUpdateFields[] = 'last_order_date';
                        }
                    }
                    if ($customer?->getOrderCount()) {
                        $enrichedCustomer['order_count'] = $customer->getOrderCount();
                        if (!in_array('order_count', $customerUpdateFields)) {
                            $customerUpdateFields[] = 'order_count';
                        }
                    }
                    if ($customer?->getOrderTotalAmount()) {
                        $enrichedCustomer['order_total_amount'] = $customer->getOrderTotalAmount();
                        if (!in_array('order_total_amount', $customerUpdateFields)) {
                            $customerUpdateFields[] = 'order_total_amount';
                        }
                    }

                    if ($customer?->getCampaignCode()) {
                        $enrichedCustomer['campaign_code'] = $customer->getCampaignCode();
                        if (!in_array('campaign_code', $customerUpdateFields)) {
                            $customerUpdateFields[] = 'campaign_code';
                        }
                    }

                    if ($customer?->getAffiliateCode()) {
                        $enrichedCustomer['affiliate_code'] = $customer->getAffiliateCode();
                        if (!in_array('affiliate_code', $customerUpdateFields)) {
                            $customerUpdateFields[] = 'affiliate_code';
                        }
                    }

                    if ($customer?->getGroup()?->getName()) {
                        $enrichedCustomer['customer_group'] = $customer->getGroup()->getName();
                        if (!in_array('customer_group', $customerUpdateFields)) {
                            $customerUpdateFields[] = 'customer_group';
                        }
                    }

                    $enrichedCustomer['_unsubscribed'] = !($subscriber?->getStatus() === "direct" || $subscriber?->getStatus() === "optIn");
                    if (!in_array('_unsubscribed', $customerUpdateFields)) {
                        $customerUpdateFields[] = '_unsubscribed';
                    }

                    $enrichedCustomer['intent'] = $subscriber?->getStatus();
                    if (!in_array('intent', $customerUpdateFields)) {
                        $customerUpdateFields[] = 'intent';
                    }

                    if ($subscriber?->getCity()) {
                        $enrichedCustomer['city'] = $subscriber->getCity();
                        $enrichedCustomer['geo_location']['city'] = $subscriber->getCity();
                        if (!in_array('city', $customerUpdateFields)) {
                            $customerUpdateFields[] = 'city';
                        }
                    }
                    if ($subscriber?->getZipCode()) {
                        $enrichedCustomer['zip_code'] = $subscriber->getZipCode();
                        if (!in_array('zip_code', $customerUpdateFields)) {
                            $customerUpdateFields[] = 'zip_code';
                        }
                    }

                    if ($customer?->getTagIds()) {
                        $enrichedCustomer['_tags'] = [];
                        foreach ($customer->getTagIds() as $tagId) {
                            if ($tag = $this->getTag($tagId)) {
                                $enrichedCustomer['_tags'][] = $tag;
                            }
                        }
                    }
                    if ($subscriber?->getTags()) {
                        if (!isset($enrichedCustomer['_tags'])) {
                            $enrichedCustomer['_tags'] = [];
                        }
                        foreach ($subscriber->getTags() as $tag) {
                            if ($tag?->getName() && !in_array($tag->getName(), $enrichedCustomer['_tags'])) {
                                $enrichedCustomer['_tags'][] = $tag->getName();
                            }
                        }
                    }

                    /* dont use default import because it relies on queue times
                    $requestData = [
                        "action" => "subscription_subscribe",
                        "json" => [$enrichedCustomer],
                        "activate_trigger" => true,
                        "default_subscribe" => true,
                        "force_subscribe" => true,
                        "update_if_exists" => true,
                        "overwrite_tags" => true,
                        "update_fields" => $customerUpdateFields
                    ];
                    $this->juneApiClient->sendData($requestData, "https://engagement.juneapp.com/collect/$commerceCollectToken/import/json", $commerceCollectAuth);
                    */
                    $this->upsertCustomer($customer, $enrichedCustomer, $commerceCollectToken, $commerceCollectAuth, $salesChannelId);
                }
            } catch (\Exception $e) {
                $this->loggerHelper->logError($e->getMessage(), "commerceNewsletterSubscribe");
            }
        }
    }

    /**
     * @param NewsletterUnsubscribeEvent|EntityDeletedEvent $event
     * @return void
     */
    public function newsletterUnsubscribe(NewsletterUnsubscribeEvent|EntityDeletedEvent $event) {
        // $newsletterRecipient = $event->getNewsletterRecipient(); // The event newsletterRecipient does not contain any tags or tagIds...
        try {
            $newsletterRecipient = $this->getNewsletterRecipient($event->getNewsletterRecipientId());
            $salesChannelId = $event->getSalesChannelId() ?? $newsletterRecipient->getSalesChannelId();
        } catch (\Exception $e) {
            $this->loggerHelper->logError($e->getMessage(), "newsletterUnsubscribe");
        }

        if ($newsletterRecipient && $salesChannelId && $this->configService->get(self::JUNE_TRANSFER_SUBSCRIBERS, $salesChannelId)) {
            try {
                $commerceCollectToken = $this->configService->get(self::JUNE_COMMERCE_COLLECT_TOKEN, $salesChannelId);
                $commerceCollectAuth = $this->configService->get(self::JUNE_COMMERCE_COLLECT_AUTH, $salesChannelId);

                if (!empty($commerceCollectToken) && !empty($commerceCollectAuth)) {
                    $updateFields = [
                        "firstname",
                        "lastname",
                        "store_id",
                        "locale",
                        "_unsubscribed",
                        "geo_location"
                    ];

                    $customer = $this->getCustomerByEmail($newsletterRecipient->getEmail());
                    $enrichedCustomer = [
                        'external_id'   => $customer?->getCustomerNumber() ?? $newsletterRecipient?->getId(),
                        'email'         => strtolower($newsletterRecipient->getEmail()),
                        'firstname'     => $newsletterRecipient->getFirstname(),
                        'lastname'      => $newsletterRecipient->getLastname(),
                        'store_id'      => $salesChannelId,
                        'locale'        => $this->getLanguageCode($event->getContext()->getLanguageId()),
                        '_unsubscribed' => true,
                        'geo_location'  => []
                    ];

                    if ($customer?->getId()) {
                        $enrichedCustomer['customer_id'] = $customer->getId();
                        $updateFields[] = 'customer_id';
                    }

                    if ($newsletterRecipient->getCity()) {
                        $updateFields[] = 'city';
                        $enrichedCustomer['city'] = $newsletterRecipient->getCity();
                        $enrichedCustomer['geo_location']['city'] = $enrichedCustomer['city'];
                    }
                    if ($newsletterRecipient->getZipCode()) {
                        $updateFields[] = 'zip_code';
                        $enrichedCustomer['zip_code'] = $newsletterRecipient->getZipCode();
                    }

                    if ($newsletterRecipient?->getTags()) {
                        $enrichedCustomer['_tags'] = [];
                        foreach ($newsletterRecipient->getTags() as $tag) {
                            if ($tag?->getName()) {
                                $enrichedCustomer['_tags'][] = $tag->getName();
                            }
                        }
                    }

                    /* dont use default import because it relies on queue times
                    $requestData = [
                        "action" => "subscription_unsubscribe",
                        "json" => [$enrichedCustomer],
                        "activate_trigger" => true,
                        "default_subscribe" => false,
                        "force_subscribe" => false,
                        "update_if_exists" => true,
                        "update_fields" => $updateFields
                    ];
                    $this->juneApiClient->sendData($requestData, "https://engagement.juneapp.com/collect/$commerceCollectToken/import/json", $commerceCollectAuth);
                    */
                    $this->upsertCustomer($customer, $enrichedCustomer, $commerceCollectToken, $commerceCollectAuth, $salesChannelId);
                }
            } catch (\Exception $e) {
                $this->loggerHelper->logError($e->getMessage(), "commerceNewsletterUnsubscribe");
            }
        }
    }

    /**
     * @param $customer
     * @param $collectData
     * @param $collectToken
     * @param $collectAuth
     * @param $salesChannelId
     * @return void
     */
    private function upsertCustomer($customer, $collectData, $collectToken, $collectAuth, $salesChannelId): void
    {
        // load JuneId if available
        $collectId = null;
        if ($customer && $customer->getId()) {
            $juneCustomer = $this->getJuneCustomerData($customer->getId(), $salesChannelId);
            $collectId = $juneCustomer?->getVars()['juneId'];
        }

        $customerResponse = $this->juneApiClient->upsertCustomer($collectData, $collectId, $collectToken, $collectAuth);

        // update customer so it is checked in the scheduled export
        if ($customer) {
            $customerUpdateData = [
                'id' => $customer->getId()
            ];

            // add juneId if available/necessary
            if (!$customer->getExtension('juneCustomer')?->getVars()['juneId'] && $customerResponse && $customerResponse['id']) {
                $customerUpdateData['juneCustomer'] = [
                    [
                        'salesChannelId' => $salesChannelId,
                        'juneId' => $customerResponse['id']
                    ]
                ];
            }

            // only set updated at and juneId or proper CustomerEntity instances
            if ($customer instanceof CustomerEntity) {
                $this->customerEntity->update([
                    $customerUpdateData
                ], Context::createDefaultContext());
            }
        }
    }

    /**
     * @param $customerId
     * @param $salesChannelId
     * @return \Shopware\Core\Framework\DataAbstractionLayer\Entity|null
     */
    private function getJuneCustomerData($customerId, $salesChannelId) {
        $searchCriteria = new Criteria();
        $searchCriteria->addFilter(new EqualsFilter('customerId', $customerId));
        $searchCriteria->addFilter(new EqualsFilter('salesChannelId', $salesChannelId));

        return $this->juneCustomerRepository->search($searchCriteria, Context::createDefaultContext())->first();
    }

    /**
     * @param string $salesChannelId
     * @return SalesChannelContext
     */
    public function createSalesChannelContext(string $salesChannelId): SalesChannelContext
    {
        return $this->salesChannelContextFactory->create('', $salesChannelId, []);
    }

    /**
     * @param $salesChannelId
     * @param $languageId
     * @return mixed|null
     */
    public function getShopUrl($salesChannelId, $languageId) {
        $salesChannelUrls = [];
        $salesCriteria = new Criteria();
        $salesCriteria = $salesCriteria->addAssociation('domains');
        $salesChannelIds = $this->salesRepository->search($salesCriteria, Context::createDefaultContext());
        foreach($salesChannelIds->getEntities()->getElements() as $key => $salesChannel){
            foreach($salesChannel->getDomains() as $element) {
                array_push($salesChannelUrls, $element);
            }
        }
        foreach ($salesChannelUrls as $url) {
            if($url->getSalesChannelId() === $salesChannelId && $url->getLanguageId() === $languageId) {
                return $url->getUrl();
            }
        }
        return null;
    }

    /**
     * @param $languageId
     * @return mixed
     */
    public function getLanguageCode($languageId) {
        $languageCriteria = new Criteria();
        $languageCriteria = $languageCriteria->addFilter(new EqualsFilter('id', $languageId));
        $language = $this->languageRepository->search($languageCriteria, Context::createDefaultContext())->first();

        $localeCriteria = new Criteria();
        $localeCriteria = $localeCriteria->addFilter(new EqualsFilter('id', $language->getLocaleId()));
        $locale = $this->localeRepository->search($localeCriteria, Context::createDefaultContext())->first();

        return $locale->getCode();
    }

    /**
     * @param $salesChannelId
     * @return CountryEntity|null
     */
    public function getCountry($salesChannelId): ?CountryEntity {
        $salesChannelCriteria = new Criteria();
        $salesChannelCriteria = $salesChannelCriteria->addFilter(new EqualsFilter('id', $salesChannelId));
        $salesChannel = $this->salesRepository->search($salesChannelCriteria, Context::createDefaultContext())->first();

        $countryCriteria = new Criteria();
        $countryCriteria = $countryCriteria->addFilter(new EqualsFilter('id', $salesChannel->getCountryId()));
        $country = $this->countryRepository->search($countryCriteria, Context::createDefaultContext())->first();

        return $country;
    }

    /**
     * @param string $orderId
     * @param Context $context
     * @return OrderEntity|null
     */
    private function getOrder(string $orderId, Context $context): ?OrderEntity
    {
        $orderCriteria = new Criteria();
        $orderCriteria = $orderCriteria->addFilter(new EqualsFilter('id', $orderId));
        $orderCriteria->addAssociation('deliveries');

        return $this->orderRepository->search($orderCriteria, $context)->first();
    }

    /**
     * @param $customerId
     * @return CustomerEntity|null
     */
    private function getCustomer($customerId): ?CustomerEntity {
        if ($customerId) {
            $customerSearch = new Criteria([$customerId]);
            $customerSearch->addAssociation('tags');
            return $this->customerEntity->search($customerSearch, Context::createDefaultContext())->first();
        }
        return null;
    }

    /**
     * @param string $email
     * @return CustomerEntity|null
     */
    private function getCustomerByEmail(string $email): ?CustomerEntity {
        if ($email) {
            $customerSearch = new Criteria();
            $customerSearch->addFilter(new EqualsFilter('email', $email));
            $customerSearch->addAssociation('tags');
            $customerSearch->addAssociation('group');
            return $this->customerEntity->search($customerSearch, Context::createDefaultContext())->first();
        }
        return null;
    }

    /**
     * @param $recipientId
     * @return NewsletterRecipientEntity|null
     */
    private function getNewsletterRecipient($recipientId): ?NewsletterRecipientEntity {
        if ($recipientId) {
            $recipientSearch = new Criteria([$recipientId]);
            $recipientSearch->addAssociation('tags');
            $recipientSearch->addAssociation('juneCustomer');
            return $this->newsletterRecipientRepository->search($recipientSearch, Context::createDefaultContext())->first();
        }
        return null;
    }

    /**
     * @param string $email
     * @return NewsletterRecipientEntity|null
     */
    private function getNewsletterRecipientByEmail(string $email): ?NewsletterRecipientEntity {
        if ($email) {
            $recipientSearch = new Criteria();
            $recipientSearch->addFilter(new EqualsFilter('email', $email));
            $recipientSearch->addAssociation('tags');
            return $this->newsletterRecipientRepository->search($recipientSearch, Context::createDefaultContext())->first();
        }
        return null;
    }

    /**
     * @param string $tagId
     * @return mixed
     */
    private function getTag(string $tagId) {
        if (!in_array($tagId, $this->loadedCustomerTags)) {
            $tag = $this->customerTagsRepository->search(new Criteria([$tagId]), Context::createDefaultContext())->first();
            $this->loadedCustomerTags[$tagId] = $tag->getName();
        }

        return $this->loadedCustomerTags[$tagId];
    }

    /**
     * @param $salutationId
     * @return \Shopware\Core\Framework\DataAbstractionLayer\Entity|null
     */
    private function getSalutation($salutationId) {
        if ($salutationId) {
            $salutationSearch = new Criteria([$salutationId]);
            return $this->salutationRepository->search($salutationSearch, Context::createDefaultContext())->first();
        }
        return null;
    }
}
